package ru.dbotthepony.mc.otm.compat.vanilla

import net.minecraft.core.BlockPos
import net.minecraft.core.registries.Registries
import net.minecraft.world.Container
import net.minecraft.world.flag.FeatureFlags
import net.minecraft.world.inventory.MenuType
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Blocks
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.ChestBlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.neoforged.bus.api.IEventBus
import net.neoforged.neoforge.capabilities.IBlockCapabilityProvider
import net.neoforged.neoforge.capabilities.RegisterCapabilitiesEvent
import net.neoforged.neoforge.client.event.RegisterMenuScreensEvent
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.capability.IQuickStackContainer
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.menu.makeSlots
import ru.dbotthepony.mc.otm.registry.MDeferredRegister

object VanillaMenuTypes {
	private val registrar = MDeferredRegister(Registries.MENU, OverdriveThatMatters.MOD_ID)

	val GENERIC_9x1 by registrar.register("generic_9x1") { MenuType(MatteryChestMenu::c9x1, FeatureFlags.VANILLA_SET) }
	val GENERIC_9x2 by registrar.register("generic_9x2") { MenuType(MatteryChestMenu::c9x2, FeatureFlags.VANILLA_SET) }
	val GENERIC_9x3 by registrar.register("generic_9x3") { MenuType(MatteryChestMenu::c9x3, FeatureFlags.VANILLA_SET) }
	val GENERIC_9x4 by registrar.register("generic_9x4") { MenuType(MatteryChestMenu::c9x4, FeatureFlags.VANILLA_SET) }
	val GENERIC_9x5 by registrar.register("generic_9x5") { MenuType(MatteryChestMenu::c9x5, FeatureFlags.VANILLA_SET) }
	val GENERIC_9x6 by registrar.register("generic_9x6") { MenuType(MatteryChestMenu::c9x6, FeatureFlags.VANILLA_SET) }
	val GENERIC_3x3 by registrar.register("generic_3x3") { MenuType(MatteryChestMenu::c3x3, FeatureFlags.VANILLA_SET) }
	val HOPPER by registrar.register("hopper") { MenuType(MatteryChestMenu::hopper, FeatureFlags.VANILLA_SET) }

	val SHULKER_BOX by registrar.register("shulker_box") { MenuType(::MatteryShulkerBoxMenu, FeatureFlags.VANILLA_SET) }

	private fun provider(level: Level, pos: BlockPos, state: BlockState, blockEntity: BlockEntity?, context: Void?): IQuickStackContainer? {
		val container = blockEntity as? Container ?: return null
		return IQuickStackContainer.Simple(makeSlots(container, ::MatteryMenuSlot))
	}

	fun registerCapabilities(event: RegisterCapabilitiesEvent) {
		event.registerBlock(
			MatteryCapability.QUICK_STACK_CONTAINER,
			::provider,
			Blocks.CHEST,
			Blocks.TRAPPED_CHEST,
			Blocks.SHULKER_BOX,
			Blocks.BARREL,
		)
	}

	internal fun register(bus: IEventBus) {
		registrar.register(bus)
		bus.addListener(this::registerScreens)
		bus.addListener(this::registerCapabilities)
	}

	private fun registerScreens(event: RegisterMenuScreensEvent) {
		event.register(GENERIC_9x1, ::VanillaChestScreen)
		event.register(GENERIC_9x2, ::VanillaChestScreen)
		event.register(GENERIC_9x3, ::VanillaChestScreen)
		event.register(GENERIC_9x4, ::VanillaChestScreen)
		event.register(GENERIC_9x5, ::VanillaChestScreen)
		event.register(GENERIC_9x6, ::VanillaChestScreen)
		event.register(GENERIC_3x3, ::VanillaChestScreen)
		event.register(HOPPER, ::VanillaChestScreen)
		event.register(SHULKER_BOX, ::VanillaChestScreen)
	}
}
